import { WebServer } from './webserver';
import { Recorder } from './recorder';
export class MockServerCore extends WebServer {
    recorder = new Recorder(16 * 1024);
    constructor(config) {
        super(config);
        for (const evt of ['message', 'broadcast']) {
            this.on(evt, (msg) => {
                const time = new Date();
                this.recorder.message(time, msg);
            });
        }
        this.on('request', (req) => {
            const start = new Date();
            const recordResponse = (res) => {
                const end = new Date();
                this.recorder.hit(start, end, req, res);
                this.removeListener('response', recordResponse);
                this.removeListener('missing', recordMissing);
            };
            const recordMissing = (res) => {
                const end = new Date();
                this.recorder.miss(start, end, req);
                this.removeListener('response', recordResponse);
                this.removeListener('missing', recordMissing);
            };
            this.addListener('response', recordResponse);
            this.addListener('missing', recordMissing);
        });
    }
}
