import * as providers from '../auth';
import { parse } from 'url';
const bearer = 'Bearer ';
export function getAuth(server, config) {
    const auth = config.auth;
    const provider = auth && providers[auth.provider];
    if (provider) {
        return (server) => {
            const connectToFeed = server.feed;
            server.feed = (handler) => {
                connectToFeed((ws, req) => {
                    const { token } = parse(req.url, true).query;
                    if (!Array.isArray(token)) {
                        const valid = provider.validateToken(auth, token);
                        if (valid) {
                            return handler(ws, req);
                        }
                    }
                    ws.close();
                });
                return server;
            };
            return server.any((req, res, next) => {
                const header = req.header('authorization');
                if (header && header.startsWith(bearer)) {
                    const token = header.substring(bearer.length);
                    const valid = provider.validateToken(auth, token);
                    if (valid) {
                        return next();
                    }
                }
                return res.sendStatus(401);
            });
        };
    }
    return (server) => server;
}
export function userLogin(server, config) {
    const auth = config.auth;
    const provider = auth && providers[auth.provider];
    return (req, res) => {
        const credentials = JSON.parse(req.body || '{}');
        const token = provider && provider.generateToken(auth, credentials);
        if (token) {
            res.json({ token });
        }
        else {
            res.sendStatus(403);
        }
    };
}
